/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: itkQuadEdgeMeshToQuadEdgeMeshFilter.txx,v $
  Language:  C++
  Date:      $Date: 2007/02/25 15:19:44 $
  Version:   $Revision: 1.2 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#ifndef __itkQuadEdgeMeshToQuadEdgeMeshFilter_txx
#define __itkQuadEdgeMeshToQuadEdgeMeshFilter_txx

namespace itk
{
// ---------------------------------------------------------------------
template< class TInputMesh, class TOutputMesh >
MeshCopy< TInputMesh, TOutputMesh >
::MeshCopy()
{
  this->Superclass::SetNumberOfRequiredInputs( 1 );
  this->Superclass::SetNumberOfRequiredOutputs( 1 );

  typename TInputMesh::Pointer out = TInputMesh::New();
  this->Superclass::SetNthOutput( 0, out.GetPointer() );
}

// ---------------------------------------------------------------------
template< class TInputMesh, class TOutputMesh >
void 
MeshCopy< TInputMesh, TOutputMesh >
::GenerateData()
{
  InputMeshConstPointer in = this->GetInput();
  OutputMeshPointer out = this->GetOutput();

  // Copy points
  InputPointsContainerConstIterator inIt = in->GetPoints()->Begin();
  while( inIt != in->GetPoints()->End() )
    {
    OutputPointType pOut;
    pOut.CastFrom( inIt.Value() );
    out->SetPoint( inIt.Index(), pOut );
    inIt++;
    } 

  // Copy cells
  InputCellsContainerConstIterator cIt = in->GetCells()->Begin();
  for( cIt != in->GetCells()->End(); cIt++ )
    {
    InputEdgeCellType* qe = (InputEdgeCellType*)0;
    InputPolygonCellType* pe = (InputPolygonCellType*)0;
    if( ( qe = dynamic_cast< InputEdgeCellType* >( cIt.Value() ) ) )
      {
      out->AddEdge( qe->GetOrg(), qe->GetDest() );
      }
    else
      {
      pe = dynamic_cast< InputPolygonCellType* >( cIt.Value());
      if( pe )
        {
        InputPointIdList points;
        typename InputPolygonCellType::PointIdIterator pit =
          pe->PointIdsBegin();
        while( pit != pe->PointIdsEnd();)
          {
          points.push_back( ( *pit ) );
          pit++;
          }
        out->AddFace( points );
        } 
      }
    cIt++;
    }
}

} // end namespace itk

#endif 
